/*
 *  shadowircd: an advanced Internet Relay Chat Daemon(ircd).
 *  m_help.c: Provides help information to a user/operator.
 *
 *  $Id: help.c,v 1.1.4.9 2004/12/29 20:58:30 nenolod Exp $
 *
 * Author's note:
 *
 * Yes. I know this module is messy. I don't particularly care at this time.
 * The first chunk after declarations is the needed elements for the
 * configuration parser (newconf). Then the module header. Then the
 * help frontend, v1. Past that comes help frontend v2. v1 tries to use the
 * v2 conf-based interface. If that fails, it uses the v1 data.
 *
 * Help entries are stored in memory, via dlink_list help_entries.
 * Lines of help data are stored in a dlink_list inside the entry.
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "ircd.h"
#include "ircd_handler.h"
#include "msg.h"
#include "numeric.h"
#include "send.h"
#include "s_conf.h"
#include "s_log.h"
#include "parse.h"
#include "modules.h"
#include "irc_string.h"
#include "hook.h"
#include "list.h"
#include "tools.h"
#include "help.h"
#include "newconf.h"

static void m_help(struct Client *, struct Client *, int, char **);
dlink_list help_entries = { NULL, NULL, 0 };
struct helpEntry *currentEntry;
static int helpLineType;

/********** Parser routines, for newconf.c **********/

static int
conf_help_start(struct TopConf *tc)
{
	currentEntry = MyMalloc(sizeof(struct helpEntry));
	helpLineType = 0;
	return 0;
}

static int
conf_help_end(struct TopConf *tc)
{
	if (conf_cur_block_name != NULL)
	{
		DupString(currentEntry->name, conf_cur_block_name);
		dlinkAddTail(currentEntry, make_dlink_node(), &help_entries);
	} else
		MyFree(currentEntry);

	return 0;
}

static void
conf_help_text(void *data)
{
	struct helpLine *line = MyMalloc(sizeof(struct helpLine));
	strcpy(line->line, (char *) data);
	line->operonly = helpLineType;
	dlinkAddTail(line, make_dlink_node(), &currentEntry->lines);
}

static void
conf_help_operonly(void *data)
{
	int yesno = *(unsigned int *) data;

	helpLineType = yesno;
}

static void
conf_help_rehash(void *unused)
{
	dlink_node *n, *n2;
	struct helpEntry *entry;

	DLINK_FOREACH_SAFE(n, n2, help_entries.head)
	{
		entry = n->data;
		MyFree(entry);
		dlinkDelete(n, &help_entries);
		MyFree(n);
	}
}

/********** Module header **********/

struct Message help_msgtab = {
	"HELP", 0, 0, 0, 0, MFLG_SLOW, 0,
	{m_unregistered, m_help, m_ignore, m_help, m_ignore}
};

struct Message ircdhelp_msgtab = {
	"IRCDHELP", 0, 0, 0, 0, MFLG_SLOW, 0,
	{m_unregistered, m_help, m_ignore, m_help, m_ignore}
};

#define help(x) sendto_one(source_p, ":%s NOTICE %s :%s", me.name, source_p->name, (x));

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&help_msgtab);
	mod_add_cmd(&ircdhelp_msgtab);
	add_top_conf("help", conf_help_start, conf_help_end);
	add_conf_item("help", "text", CF_QSTRING, conf_help_text);
	add_conf_item("help", "operonly", CF_YESNO, conf_help_operonly);
	hook_add_hook("pre_rehash", (hookfn *) conf_help_rehash);
}

void
_moddeinit(void)
{
	mod_del_cmd(&help_msgtab);
	mod_del_cmd(&ircdhelp_msgtab);
	remove_conf_item("help", "text");
	remove_conf_item("help", "operonly");
	remove_top_conf("help");
	hook_del_hook("pre_rehash", (hookfn *) conf_help_rehash);
}

const char *_version = "$Revision: 1.1.4.9 $";
#endif

/********* Help v2 routines *********/

static struct helpEntry *
find_help_entry(char *name)
{
	dlink_node *node;
	struct helpEntry *data;

	DLINK_FOREACH(node, help_entries.head)
	{
		data = node->data;
		if (!strcasecmp(data->name, name))
			return data;
	}

	return NULL;
}

static void
send_help_entry(struct Client *source_p, struct helpEntry *entry)
{
	dlink_node *node;
	struct helpLine *line;

	DLINK_FOREACH(node, entry->lines.head)
	{
		line = node->data;
		if ((line->operonly && IsOper(source_p)) || !(line->operonly))
			help(line->line);
	}
}

/*
 * m_help - HELP message handler
 *      parv[0] = sender prefix
 */
static void
m_help(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	struct helpEntry *entry;

	if (!parv[1])
	{
		entry = find_help_entry("index");
		if (entry)
			send_help_entry(source_p, entry);
		return;
	}

	entry = find_help_entry(parv[1]);
	if (entry)
	{
		send_help_entry(source_p, entry);
		return;
	}

	if (!strcasecmp(parv[1], "chanmodes"))
	{
		help("Channel modes:");
		help(" ");
		help("Channel modes are set via /mode <channel> +/-<modes> [params].");
		if (IsOper(source_p))
			help("\2Modes in BOLD are Operator-only.\2");
		help(" ");
		help("+b - Channel Ban");
		help("+c - Deny colors");
		help("+d - Channel deny (clients matching these masks can't get halfops or higher)");
		help("+e - Channel Ban Exception");
		help("+h - Channel Halfop");
		help("+i - Invite only");
		help("+k - Channel key");
		help("+l - Channel limit");
		help("+m - Moderation (only those with status can speak)");
		help("+n - No external messages");
		help("+o - Channel operator");
		help("+p - Paranoia mode (extended privilege seperation)");
		help("+q - Channel quiet (mask-based moderation)");
		help("+r - Channel is registered with services");
		help("+s - Channel is not to appear in /list");
		help("+t - Channel's topic is locked to halfops or higher.");
		help("+u - Channel admin");
		help("+v - Channel voice");
		help("+z - Disable channel flood protection");
		help("+A - Anonymous Prefixing (RFC2812)");
		help("+E - Registered nicks only");
		help("+F - Anyone can invite users to the channel");
		help("+G - Strip badwords from messages sent to channel");
		help("+I - Channel Invite Exception (mask doesnt have to be invited when channel is +i)");
		help("+K - Disable KNOCK on channel");
		if (IsOper(source_p))
			help("\2+L - Remove banlist limits on channel\2");
		help("+N - Disable ability of users on channel to change nick");
		if (IsOper(source_p))
			help("\2+O - Operator-only channel\2");
		help("+P - Channel is in peace mode -- locked down to channel admins only.");
		if (IsOper(source_p))
			help("\2+R - Channel is permanant, can't be closed\2");
		help("+S - Strip colors from messages sent to channel");
		help("+T - Topic is locked to channel admins only.");
		help("+V - Anyone can voice themselves in the channel.");
		help(" ");
		help("This is a work in progress!");
		help("For more help, see http://www.shadowircd.net/?userhelp");
		help("(That page may or may not yet exist.)");
	}

	if (!strcasecmp(parv[1], "commands"))
	{
		list_commands(source_p);
		return;
	}

	help("There is no available help on this topic.");
}
