/*
 *  shadowircd: an advanced Internet Relay Chat Daemon(ircd).
 *  svspart.c: Parts a user from a channel.
 *
 *  $Id: svspart.c,v 1.1.2.1 2004/12/27 22:05:59 nenolod Exp $
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "client.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "msg.h"
#include "modules.h"
#include "parse.h"
#include "hash.h"
#include "packet.h"
#include "s_serv.h"

static void m_svspart(struct Client *, struct Client *, int, char **);

struct Message svspart_msgtab = {
	"SVSPART", 0, 0, 2, 0, MFLG_SLOW, 0,
	{m_unregistered, m_ignore, m_svspart, m_ignore, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&svspart_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&svspart_msgtab);
}

const char *_version = "$Revision: 1.1.2.1 $";
const char *_desc =
    "Implements SVSPART services command -- removes a user from a channel.";
#endif

/* m_svspart()
 *  parv[0] = sender prefix
 *  parv[1] = client
 *  parv[2] = channel
 *  parv[3] = part comment
 */
static void
m_svspart(struct Client *client_p, struct Client *source_p, int parc,
    char *parv[])
{
	struct Client *who;
	struct Channel *chptr;
	struct Membership *ms;

	who = find_client(parv[1]);

	if ((chptr = hash_find_channel(parv[2])) == NULL)
		return;

	if ((ms = find_channel_link(who, chptr)) == NULL)
		return;

	sendto_channel_local(ALL_MEMBERS, chptr,
	    ":%s!%s@%s PART %s :%s", source_p->name,
	    source_p->username, GET_CLIENT_HOST(source_p), parv[2],
	    parv[3]);

	sendto_server(client_p, PROTO_NEW, PROTO_CUR,
	    ":%s SVSPART %s %s :%s", ID(source_p), ID(who), chptr->chname,
	    parv[3]);

	sendto_server(client_p, PROTO_NUL, PROTO_MIN,
	    ":%s SVSPART %s %s :%s", source_p->name, who->name, chptr->chname,
	    parv[3]);

	remove_user_from_channel(ms);
}
