/*
 *  shadowircd: an advanced Internet Relay Chat Daemon(ircd).
 *  tssync.c: Syncs the network's time.
 *
 *  Copyright (C) 2002 by the past and present ircd coders, and others.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: tssync.c,v 1.1.2.7 2004/12/01 15:30:02 nenolod Exp $
 */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"		/* TRUE bleah */
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "s_conf.h"
#include "s_log.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

static void mo_tssync(struct Client *, struct Client *, int, char **);
static void ms_tssync(struct Client *, struct Client *, int, char **);

struct Message tssync_msgtab = {
	"TSSYNC", 0, 0, 0, 0, MFLG_SLOW, 0,
	{m_unregistered, m_ignore, ms_tssync, mo_tssync, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&tssync_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&tssync_msgtab);
}

const char *_version = "$Revision: 1.1.2.7 $";
#endif
/*
 * ms_tssync - TSSYNC message handler
 *      parv[0] = sender prefix
 *      parv[1] = server's idea of UTC time
 */
static void
ms_tssync(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	time_t deltat;
	time_t theirtime;

	set_time();
	theirtime = atol(parv[1]);
	deltat = theirtime - RealTime;

	/* Protocol 3402 has TSSYNC, deal with it here. */
	if (client_p->protocol > PROTO_NEW)
	{
		ts_offset = deltat;
		sendto_server(client_p, PROTO_CUR, PROTO_CUR, ":%s TSSYNC %lu",
				     me.id, CurrentTime);
	}
}

/*
 * ms_tssync - TSSYNC message handler
 *      parv[0] = sender prefix
 *      parv[1] = server's idea of UTC time
 */
static void
mo_tssync(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	int res;

	if (parv[1])
	{
		ts_offset = (RealTime - atol(parv[1]));
		sendto_server(NULL, PROTO_CUR, PROTO_CUR, ":%s TSSYNC %lu",
				me.id, CurrentTime);
	}
	res = CurrentTime - RealTime;
	sendto_one(source_p, ":%s NOTICE %s :*** CurrentTime: %lu - RealTime %lu = %d",
				me.name, source_p->name, CurrentTime, RealTime,
				res);
}

